@@ -72,7 +72,7 @@ SEXP simulation_computations (SEXP object, SEXP params, SEXP times, SEXP t0,
   
   for (j = 1; j < ntimes; j++) { // copy times[2:ntimes] into alltimes[3:(ntimes+1)]
     if (tt >= *s)
-      errorcall(R_NilValue,"'times' must be an increasing sequence of times");
+      errorcall(R_NilValue,"'times' must be an increasing sequence");
     tt = *(t++) = *(s++);
   }
 
