@@ -116,7 +116,7 @@ static void SSA (pomp_ssa_rate_fn *ratefun, int irep,
   double tmax = times[ntimes-1];
   double *covars = NULL;
   double *f = NULL;
-  double par[npar], y[nvar];
+  double par[npar], y[nvar], ynext[nvar];
   struct lookup_table tab = {lcov, mcov, 0, tcov, cov};
   int i, j;
 
@@ -124,16 +124,17 @@ static void SSA (pomp_ssa_rate_fn *ratefun, int irep,
   if (nevent > 0) f = (double *) Calloc(nevent,double);
 
   // Copy parameters and states
-  for (i = 0; i < npar; i++) par[i] = params[i+npar*irep];
-  for (i = 0; i < nvar; i++)
-    xout[i+irep*nvar] = y[i] = xstart[i+nvar*irep];
+  memcpy(par,params+npar*irep,npar*sizeof(double));
+  memcpy(y,xstart+nvar*irep,nvar*sizeof(double));
+  memcpy(xout+nvar*irep,xstart+nvar*irep,nvar*sizeof(double));
   // Set appropriate states to zero
   for (i = 0; i < nzero; i++) y[izero[i]] = 0.0;
+  memcpy(ynext,y,nvar*sizeof(double));
   // Initialize the covariate vector
   if (mcov > 0) table_lookup(&tab,t,covars);
   // Initialise propensity functions & tree
   for (j = 0; j < nevent; j++) {
-    f[j] = ratefun(j+1,t,y,par,istate,ipar,icovar,mcov,covars);
+    f[j] = ratefun(j+1,t,ynext,par,istate,ipar,icovar,mcov,covars);
     if (f[j] < 0.0)
       errorcall(R_NilValue,"'rate.fun' returns a negative rate");
   }
@@ -141,33 +142,40 @@ static void SSA (pomp_ssa_rate_fn *ratefun, int irep,
   while (icount < ntimes) {
     R_CheckUserInterrupt();
     if (method == 0) {	// Gillespie's next reaction method
-      flag = gillespie(ratefun,&t,f,y,v,d,par,nvar,nevent,npar,istate,ipar,ncovar,icovar,mcov,covars);
+      flag = gillespie(ratefun,&t,f,ynext,v,d,par,nvar,nevent,npar,istate,ipar,
+        ncovar,icovar,mcov,covars);
     } else {	 // Cai's K-leap method
       // Determine kappa (most accurate but slowest method)
       double kappa, tmp;
       int k;
       for (i = 0, kappa = 1e9; i < ndeps; i++) {
         k = ideps[i];
-        tmp = e[k]*y[k];
+        tmp = e[k]*ynext[k];
         kappa = (tmp < kappa) ? tmp : kappa;
         if (kappa < 2.0) break;
       }
       if (kappa < 2.0) {
-        flag = gillespie(ratefun,&t,f,y,v,d,par,nvar,nevent,npar,istate,ipar,ncovar,icovar,mcov,covars);
+        flag = gillespie(ratefun,&t,f,ynext,v,d,par,nvar,nevent,npar,istate,
+          ipar,ncovar,icovar,mcov,covars);
       } else {
         kappa = floor(kappa);
-        flag = kleap(ratefun,kappa,&t,f,y,v,d,par,nvar,nevent,npar,istate,ipar,ncovar,icovar,mcov,covars);
+        flag = kleap(ratefun,kappa,&t,f,ynext,v,d,par,nvar,nevent,npar,istate,
+          ipar,ncovar,icovar,mcov,covars);
       }
     }
+
     // Record output at required time points
-    while ((icount < ntimes) && (t >= times[icount])) {
-      for (i = 0; i < nvar; i++)
-        xout[i+nvar*(irep+nrep*icount)] = y[i];
-      // Set appropriate states to zero
-      for (i = 0; i < nzero; i++) y[izero[i]] =0.0;
-      // Recompute if zero event-rate encountered
-      if (flag) t = times[icount];
-      icount++;
+    if (t >= times[icount]) {
+      while ((t >= times[icount]) && (icount < ntimes)) {
+        memcpy(xout+nvar*(irep+nrep*icount),y,nvar*sizeof(double));
+        // Set appropriate states to zero at time of last observation
+        for (i = 0; i < nzero; i++) y[izero[i]] = 0;
+        // Recompute if zero event-rate encountered
+        if (flag) t = times[icount];
+        icount++;
+      }
+      memcpy(y,ynext,nvar*sizeof(double));
+      for (i = 0; i < nzero; i++) ynext[izero[i]] = 0;
     }
 
     if ((mcov > 0) && (t <= tmax)) table_lookup(&tab,t,covars);
